﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Speech.Recognition;
using System.Globalization;

namespace Recognition
{
    public partial class FrmMain : Form
    {
        private SpeechRecognitionEngine recognizer;  //语音识别引擎
        private string engineId = "MS-2052-80-DESK"; //Win7下语音识别引擎编号
        private string currentCulture = "zh-CN"; //语言

        public FrmMain()
        {
            InitializeComponent();
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            /*
             * 如果系统存在多个识别引擎，则初始化指定编号和语言的引擎
             * 关于其它引擎的编号可以从SpeechRecognitionEngine.InstalledRecognizers()中获取
             */
            CultureInfo cultureInfo = new CultureInfo(currentCulture); //当前地区文化
            foreach (RecognizerInfo recognizerInfo in SpeechRecognitionEngine.InstalledRecognizers())
            {
                if (recognizerInfo.Culture.Equals(cultureInfo) && recognizerInfo.Id == engineId)
                {
                    recognizer = new SpeechRecognitionEngine(recognizerInfo);
                    recognizer.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(Recognizer_SpeechRecognized);
                    break;
                }
            }

            //卸载所有语法
            recognizer.UnloadAllGrammars();

            //加载自然语法
            DictationGrammar dictationGrammar = new DictationGrammar();
            recognizer.LoadGrammar(dictationGrammar);
        }

        /// <summary>
        /// 开始聆听
        /// </summary>
        private void btnStart_Click(object sender, EventArgs e)
        {
            //设置语音输入设备(使用系统默认输入设备)
            recognizer.SetInputToDefaultAudioDevice();

            //启用连续语音识别模式
            recognizer.RecognizeAsync(RecognizeMode.Multiple);

            this.btnStart.Text = "聆听中...";
            this.btnStart.Enabled = false;
            this.btnStop.Enabled = true;
        }

        /// <summary>
        /// 停止聆听
        /// </summary>
        private void btnStop_Click(object sender, EventArgs e)
        {
            recognizer.RecognizeAsyncStop();

            this.btnStart.Text = "开始聆听";
            this.btnStart.Enabled = true;
            this.btnStop.Enabled = false;
        }

        /// <summary>
        /// 连续识别完成时引发
        /// </summary>
        private void Recognizer_SpeechRecognized(Object sender, SpeechRecognizedEventArgs e)
        {
            this.rtxtResult.Text = e.Result.Text;
        }
    }
}
